@@ -1,7 +1,7 @@
 //! Provides high-level access to the filesystem APIs.
 
 use crate::error::Error;
-use crate::helpers::{Ignore, OneValue, TwoValues};
+use crate::helpers::{Ignore, OneOptionalValue, OneValue, TwoValues};
 use alloc::borrow::ToOwned;
 use alloc::vec::Vec;
 use minicbor::{Decode, Encode};
@@ -81,7 +81,7 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	/// * [`BadComponent`](Error::BadComponent) is returned if the filesystem does not exist, is
 	///   inaccessible, or is not a filesystem.
 	pub async fn get_label(self) -> Result<Option<&'buffer str>, Error> {
-		let ret: OneValue<Option<&'buffer str>> =
+		let ret: OneValue<_> =
 			component_method::<(), _>(self.invoker, self.buffer, &self.address, "getLabel", None)
 				.await?;
 		Ok(ret.0)
@@ -98,15 +98,14 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	/// * [`Failed`](Error::Failed) is returned if the drive does not support labels or if the
 	///   label cannot be changed.
 	pub async fn set_label(self, label: Option<&str>) -> Result<Option<&'buffer str>, Error> {
-		let ret: Result<OneValue<Option<&'buffer str>>, oc_wasm_safe::error::Error> =
-			component_method(
-				self.invoker,
-				self.buffer,
-				&self.address,
-				"setLabel",
-				Some(&OneValue(label)),
-			)
-			.await;
+		let ret: Result<OneValue<_>, oc_wasm_safe::error::Error> = component_method(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"setLabel",
+			Some(&OneValue(label)),
+		)
+		.await;
 		if let Err(oc_wasm_safe::error::Error::BadParameters) = ret {
 			// This is returned if the filesystem has a label but the label cannot be modified, as
 			// happens for a tmpfs.
@@ -122,7 +121,7 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	/// * [`BadComponent`](Error::BadComponent) is returned if the filesystem does not exist, is
 	///   inaccessible, or is not a filesystem.
 	pub async fn is_read_only(&mut self) -> Result<bool, Error> {
-		let ret: OneValue<bool> =
+		let ret: OneValue<_> =
 			component_method::<(), _>(self.invoker, self.buffer, &self.address, "isReadOnly", None)
 				.await?;
 		Ok(ret.0)
@@ -167,7 +166,7 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	/// * [`BadComponent`](Error::BadComponent) is returned if the filesystem does not exist, is
 	///   inaccessible, or is not a filesystem.
 	pub async fn get_space_used(&mut self) -> Result<u64, Error> {
-		let ret: OneValue<u64> =
+		let ret: OneValue<_> =
 			component_method::<(), _>(self.invoker, self.buffer, &self.address, "spaceUsed", None)
 				.await?;
 		Ok(ret.0)
@@ -225,7 +224,7 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   inaccessible, or is not a filesystem.
 	/// * [`Failed`](Error::Failed) is returned if the path does not exist or is a file.
 	pub async fn list(self, path: &str) -> Result<Vec<DirectoryEntry<'buffer>>, Error> {
-		let ret: OneValue<Option<Vec<DirectoryEntry<'buffer>>>> = component_method(
+		let ret: OneOptionalValue<Vec<DirectoryEntry<'buffer>>> = component_method(
 			self.invoker,
 			self.buffer,
 			&self.address,
@@ -353,7 +352,7 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   inaccessible, or is not a filesystem.
 	/// * [`Failed`](Error::Failed) is returned if the method fails.
 	async fn call_open(&mut self, path: &str, mode: &str) -> Result<descriptor::Owned, Error> {
-		let descriptor: Result<OneValue<Option<descriptor::Decoded>>, oc_wasm_safe::error::Error> =
+		let descriptor: Result<OneValue<descriptor::Decoded>, oc_wasm_safe::error::Error> =
 			component_method(
 				self.invoker,
 				self.buffer,
@@ -363,15 +362,12 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 			)
 			.await;
 		match descriptor {
-			Ok(OneValue(Some(descriptor))) => Ok(
+			Ok(OneValue(descriptor)) => Ok(
 				// SAFETY: This descriptor was just generated by the open() method call, so it must
 				// be fresh and unique.
 				unsafe { descriptor.into_owned() },
 			),
-			Ok(OneValue(None)) => Err(Error::Failed("file not found".to_owned())),
-			Err(oc_wasm_safe::error::Error::Other) => {
-				Err(Error::Failed("too many handles".to_owned()))
-			}
+			Err(oc_wasm_safe::error::Error::Other) => Err(Error::Failed("open failed".to_owned())),
 			Err(e) => Err(e.into()),
 		}
 	}
@@ -563,9 +559,7 @@ impl<'handle, 'invoker, 'buffer> LockedReadHandle<'handle, 'invoker, 'buffer> {
 		match ret {
 			Ok(OneValue(Some(bytes))) => Ok(Some(bytes)),
 			Ok(OneValue(None)) => Ok(None),
-			Err(oc_wasm_safe::error::Error::Other) => {
-				Err(Error::Failed("not enough energy".to_owned()))
-			}
+			Err(oc_wasm_safe::error::Error::Other) => Err(Error::Failed("read failed".to_owned())),
 			Err(e) => Err(e.into()),
 		}
 	}
@@ -667,9 +661,7 @@ impl<'handle, 'invoker, 'buffer> LockedWriteHandle<'handle, 'invoker, 'buffer> {
 		.await;
 		match ret {
 			Ok(_) => Ok(()),
-			Err(oc_wasm_safe::error::Error::Other) => {
-				Err(Error::Failed("not enough energy".to_owned()))
-			}
+			Err(oc_wasm_safe::error::Error::Other) => Err(Error::Failed("write failed".to_owned())),
 			Err(e) => Err(e.into()),
 		}
 	}
@@ -696,12 +688,12 @@ async fn seek_impl(
 		#[n(1)] Seek,
 		#[n(2)] i64,
 	);
-	let ret: Result<OneValue<u64>, oc_wasm_safe::error::Error> = component_method(
+	let ret: Result<OneValue<_>, oc_wasm_safe::error::Error> = component_method(
 		invoker,
 		buffer,
-		&address,
+		address,
 		"seek",
-		Some(&Params(&descriptor, basis, offset)),
+		Some(&Params(descriptor, basis, offset)),
 	)
 	.await;
 	match ret {
