@@ -1,6 +1,6 @@
 //! Provides high-level access to the GPU APIs.
 
-use crate::common::{Dimension, Point, Vector2};
+use crate::common::{Dimension, Point, Rgb, Vector2};
 use crate::error::Error;
 use crate::helpers::{FiveValues, Ignore, NullAndStringOr, OneValue, TwoValues};
 use alloc::borrow::ToOwned;
@@ -95,7 +95,7 @@ impl<'a> Locked<'a> {
 	///   inaccessible, or is not a GPU.
 	#[must_use = "This function is only useful for its return value"]
 	pub async fn get_screen(&mut self) -> Result<Option<Address>, Error> {
-		let ret: OneValue<Option<Address>> =
+		let ret: OneValue<_> =
 			component_method::<(), _>(self.invoker, self.buffer, &self.address, "getScreen", None)
 				.await?;
 		Ok(ret.0)
@@ -230,7 +230,7 @@ impl<'a> Locked<'a> {
 	/// * [`Failed`](Error::Failed) is returned if the GPU is unbound.
 	#[must_use = "This function is only useful for its return value"]
 	pub async fn max_depth(&mut self) -> Result<u8, Error> {
-		let ret: NullAndStringOr<'_, OneValue<u8>> =
+		let ret: NullAndStringOr<'_, OneValue<_>> =
 			component_method::<(), _>(self.invoker, self.buffer, &self.address, "maxDepth", None)
 				.await?;
 		let ret = ret.into_result()?;
@@ -245,7 +245,7 @@ impl<'a> Locked<'a> {
 	/// * [`Failed`](Error::Failed) is returned if the GPU is unbound.
 	#[must_use = "This function is only useful for its return value"]
 	pub async fn get_depth(&mut self) -> Result<u8, Error> {
-		let ret: NullAndStringOr<'_, OneValue<u8>> =
+		let ret: NullAndStringOr<'_, OneValue<_>> =
 			component_method::<(), _>(self.invoker, self.buffer, &self.address, "getDepth", None)
 				.await?;
 		let ret = ret.into_result()?;
@@ -365,8 +365,7 @@ impl<'a> Locked<'a> {
 					Err(Error::BadComponent(oc_wasm_safe::error::Error::CborDecode))
 				}
 			}
-			Err(oc_wasm_safe::error::Error::BadParameters)
-			| Err(oc_wasm_safe::error::Error::Other) => {
+			Err(oc_wasm_safe::error::Error::BadParameters | oc_wasm_safe::error::Error::Other) => {
 				// BadParameters probably should be returned if the point is out of range. However,
 				// OpenComputers throws IndexOutOfBoundsException, not IllegalArgumentException, in
 				// this case, which maps to Other.
@@ -517,8 +516,7 @@ impl<'a> Locked<'a> {
 				ret.into_result()?;
 				Ok(())
 			}
-			Err(oc_wasm_safe::error::Error::BadParameters)
-			| Err(oc_wasm_safe::error::Error::Other) => {
+			Err(oc_wasm_safe::error::Error::BadParameters | oc_wasm_safe::error::Error::Other) => {
 				// BadParameters probably should be returned if the character involves more than
 				// one UTF-16 code unit. However, OpenComputers throws unqualified Exception, not
 				// IllegalArgumentException, in this case, which maps to Other.
@@ -611,7 +609,7 @@ impl<'a> Locked<'a> {
 		parameter: Dimension,
 		out_of_range_string: &str,
 	) -> Result<bool, Error> {
-		let ret: Result<NullAndStringOr<'_, OneValue<bool>>, _> = component_method(
+		let ret: Result<NullAndStringOr<'_, OneValue<_>>, _> = component_method(
 			self.invoker,
 			self.buffer,
 			&self.address,
@@ -639,10 +637,6 @@ impl<'a> Debug for Locked<'a> {
 	}
 }
 
-/// A 24-bit RGB colour.
-#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-pub struct Rgb(pub u32);
-
 /// A palette index.
 #[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 pub struct PaletteIndex(pub u32);
