@@ -6,7 +6,7 @@
 //! are not supported), and database interaction APIs are not supported. They may be added in a
 //! future version.
 
-use crate::common::AbsoluteSide;
+use crate::common::Side;
 use crate::error::Error;
 use crate::helpers::{
 	FiveValues, FourValues, Ignore, NullAndStringOr, OneValue, ThreeValues, TwoValues,
@@ -18,12 +18,15 @@ use minicbor::{Decode, Decoder, Encode};
 use oc_wasm_futures::invoke::{component_method, value_method};
 use oc_wasm_safe::{component::Invoker, descriptor, Address};
 
-/// The type name for inventory controller components, which can read solid inventory contents but
-/// not move items around nor do anything with fluids.
+/// The type name for inventory controller components, which can read solid inventory contents and
+/// move items around using an internal inventory (for robots and drones only, not adapters), but
+/// cannot operate on fluids.
 pub const INVENTORY_CONTROLLER_TYPE: &str = "inventory_controller";
 
-/// The type name for tank controller components, which can read fluid tank contents but not move
-/// fluids around nor do anything with items.
+/// The type name for tank controller components, which can read fluid tank contents and move
+/// fluids around using internal tanks (for robots and drones only, not adapters), but cannot
+/// operate on solid items other than moving fluid into and out of containers in the internal
+/// inventory (for robots and drones only, not adapters).
 pub const TANK_CONTROLLER_TYPE: &str = "tank_controller";
 
 /// The type name for transposer components, which can read both solid inventory and fluid tank
@@ -77,6 +80,11 @@ impl Controller {
 /// dropped to return the borrow of the invoker and buffer to the caller so they can be reused for
 /// other purposes.
 ///
+/// Where a function is declared as taking [`impl Side`](Side), an
+/// [`AbsoluteSide`](super::common::AbsoluteSide) must be passed if operating on a transposer or an
+/// upgrade module installed in an adapter, while a [`RelativeSide`](super::common::RelativeSide)
+/// must be passed if operating on an upgrade module installed in a robot or drone.
+///
 /// The `'invoker` lifetime is the lifetime of the invoker. The `'buffer` lifetime is the lifetime
 /// of the buffer.
 pub struct Locked<'invoker, 'buffer> {
@@ -91,6 +99,8 @@ pub struct Locked<'invoker, 'buffer> {
 }
 
 impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
+	// WorldInventoryAnalytics
+
 	/// Returns the number of slots in an inventory.
 	///
 	/// # Errors
@@ -98,13 +108,14 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   neither a transposer nor an inventory controller upgrade.
 	/// * [`Failed`](Error::Failed) is returned if there is not an accessible inventory on the
 	///   specified side.
-	pub async fn get_inventory_size(&mut self, side: AbsoluteSide) -> Result<u32, Error> {
+	pub async fn get_inventory_size(&mut self, side: impl Side) -> Result<u32, Error> {
+		let side: u8 = side.into();
 		let ret: NullAndStringOr<'_, OneValue<_>> = component_method(
 			self.invoker,
 			self.buffer,
 			&self.address,
 			"getInventorySize",
-			Some(&OneValue(u8::from(side))),
+			Some(&OneValue(side)),
 		)
 		.await?;
 		Ok(ret.into_result()?.0)
@@ -121,11 +132,12 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   specified side, or if the requested slot number is greater than the inventory size.
 	pub async fn get_slot_stack_size(
 		&mut self,
-		side: AbsoluteSide,
+		side: impl Side,
 		slot: NonZeroU32,
 	) -> Result<u32, Error> {
+		let side: u8 = side.into();
 		let ret: OneValue<_> = self
-			.call_check_invalid_slots("getSlotStackSize", &TwoValues(u8::from(side), slot))
+			.call_check_invalid_slots("getSlotStackSize", &TwoValues(side, slot))
 			.await?;
 		Ok(ret.0)
 	}
@@ -142,11 +154,12 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   specified side, or if the requested slot number is greater than the inventory size.
 	pub async fn get_slot_max_stack_size(
 		&mut self,
-		side: AbsoluteSide,
+		side: impl Side,
 		slot: NonZeroU32,
 	) -> Result<Option<NonZeroU32>, Error> {
+		let side: u8 = side.into();
 		let ret: OneValue<_> = self
-			.call_check_invalid_slots("getSlotMaxStackSize", &TwoValues(u8::from(side), slot))
+			.call_check_invalid_slots("getSlotMaxStackSize", &TwoValues(side, slot))
 			.await?;
 		Ok(NonZeroU32::new(ret.0))
 	}
@@ -168,15 +181,16 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   size.
 	pub async fn compare_stacks(
 		&mut self,
-		side: AbsoluteSide,
+		side: impl Side,
 		slot_a: NonZeroU32,
 		slot_b: NonZeroU32,
 		check_nbt: bool,
 	) -> Result<bool, Error> {
+		let side: u8 = side.into();
 		let ret: OneValue<_> = self
 			.call_check_invalid_slots(
 				"compareStacks",
-				&FourValues(u8::from(side), slot_a, slot_b, check_nbt),
+				&FourValues(side, slot_a, slot_b, check_nbt),
 			)
 			.await?;
 		Ok(ret.0)
@@ -199,15 +213,13 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   size.
 	pub async fn are_stacks_equivalent(
 		&mut self,
-		side: AbsoluteSide,
+		side: impl Side,
 		slot_a: NonZeroU32,
 		slot_b: NonZeroU32,
 	) -> Result<bool, Error> {
+		let side: u8 = side.into();
 		let ret: OneValue<_> = self
-			.call_check_invalid_slots(
-				"areStacksEquivalent",
-				&ThreeValues(u8::from(side), slot_a, slot_b),
-			)
+			.call_check_invalid_slots("areStacksEquivalent", &ThreeValues(side, slot_a, slot_b))
 			.await?;
 		Ok(ret.0)
 	}
@@ -228,16 +240,17 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   reading full item information is disabled in the configuration file.
 	pub async fn get_stack_in_slot(
 		self,
-		side: AbsoluteSide,
+		side: impl Side,
 		slot: NonZeroU32,
-	) -> Result<ItemStack<'buffer>, Error> {
+	) -> Result<Option<ItemStack<'buffer>>, Error> {
+		let side: u8 = side.into();
 		let ret: Result<NullAndStringOr<'_, OneValue<_>>, oc_wasm_safe::error::Error> =
 			component_method(
 				self.invoker,
 				self.buffer,
 				&self.address,
 				"getStackInSlot",
-				Some(&TwoValues(u8::from(side), slot)),
+				Some(&TwoValues(side, slot)),
 			)
 			.await;
 		Ok(Self::unpack_bad_parameters_with_message(ret, "invalid slot")?.0)
@@ -251,13 +264,14 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	/// * [`Failed`](Error::Failed) is returned if there is not an accessible inventory on the
 	///   specified side, or if reading full item information is disabled in the configuration
 	///   file.
-	pub async fn get_all_stacks(&mut self, side: AbsoluteSide) -> Result<Snapshot, Error> {
+	pub async fn get_all_stacks(&mut self, side: impl Side) -> Result<Snapshot, Error> {
+		let side: u8 = side.into();
 		let ret: NullAndStringOr<'_, OneValue<descriptor::Decoded>> = component_method(
 			self.invoker,
 			self.buffer,
 			&self.address,
 			"getAllStacks",
-			Some(&OneValue(u8::from(side))),
+			Some(&OneValue(side)),
 		)
 		.await?;
 		let descriptor = ret.into_result()?.0;
@@ -280,18 +294,21 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	/// * [`Failed`](Error::Failed) is returned if there is not an accessible inventory on the
 	///   specified side, or if reading full item information is disabled in the configuration
 	///   file.
-	pub async fn get_inventory_name(self, side: AbsoluteSide) -> Result<&'buffer str, Error> {
+	pub async fn get_inventory_name(self, side: impl Side) -> Result<&'buffer str, Error> {
+		let side: u8 = side.into();
 		let ret: NullAndStringOr<'_, OneValue<_>> = component_method(
 			self.invoker,
 			self.buffer,
 			&self.address,
 			"getInventoryName",
-			Some(&OneValue(u8::from(side))),
+			Some(&OneValue(side)),
 		)
 		.await?;
 		Ok(ret.into_result()?.0)
 	}
 
+	// InventoryTransfer
+
 	/// Moves items between two inventories.
 	///
 	/// The `count` value indicates the maximum number of items to move. It is clamped to 64; even
@@ -310,19 +327,21 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   not a transposer.
 	/// * [`Failed`](Error::Failed) is returned if there is not an accessible inventory on one of
 	///   the specified sides, or if there is not enough energy to perform the operation.
-	pub async fn transfer_item(
+	pub async fn transfer_item<SideType: Side>(
 		&mut self,
-		source: AbsoluteSide,
-		sink: AbsoluteSide,
+		source: SideType,
+		sink: SideType,
 		count: u32,
 	) -> Result<u32, Error> {
+		let source: u8 = source.into();
+		let sink: u8 = sink.into();
 		let ret: Result<NullAndStringOr<'_, OneValue<_>>, oc_wasm_safe::error::Error> =
 			component_method(
 				self.invoker,
 				self.buffer,
 				&self.address,
 				"transferItem",
-				Some(&ThreeValues(u8::from(source), u8::from(sink), count)),
+				Some(&ThreeValues(source, sink, count)),
 			)
 			.await;
 		Ok(Locked::unpack_bad_parameters_with_message(ret, "invalid slot")?.0)
@@ -346,25 +365,22 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	/// * [`Failed`](Error::Failed) is returned if there is not an accessible inventory on one of
 	///   the specified sides, if the requested slot number is greater than the inventory size, or
 	///   if there is not enough energy to perform the operation.
-	pub async fn transfer_item_from_slot(
+	pub async fn transfer_item_from_slot<SideType: Side>(
 		&mut self,
-		source: AbsoluteSide,
-		sink: AbsoluteSide,
+		source: SideType,
+		sink: SideType,
 		count: u32,
 		source_slot: NonZeroU32,
 	) -> Result<u32, Error> {
+		let source: u8 = source.into();
+		let sink: u8 = sink.into();
 		let ret: Result<NullAndStringOr<'_, OneValue<_>>, oc_wasm_safe::error::Error> =
 			component_method(
 				self.invoker,
 				self.buffer,
 				&self.address,
 				"transferItem",
-				Some(&FourValues(
-					u8::from(source),
-					u8::from(sink),
-					count,
-					source_slot.get(),
-				)),
+				Some(&FourValues(source, sink, count, source_slot.get())),
 			)
 			.await;
 		Ok(Locked::unpack_bad_parameters_with_message(ret, "invalid slot")?.0)
@@ -385,14 +401,16 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	/// * [`Failed`](Error::Failed) is returned if there is not an accessible inventory on one of
 	///   the specified sides, if a requested slot number is greater than the corresponding
 	///   inventory size, or if there is not enough energy to perform the operation.
-	pub async fn transfer_item_from_slot_to_slot(
+	pub async fn transfer_item_from_slot_to_slot<SideType: Side>(
 		&mut self,
-		source: AbsoluteSide,
-		sink: AbsoluteSide,
+		source: SideType,
+		sink: SideType,
 		count: u32,
 		source_slot: NonZeroU32,
 		sink_slot: NonZeroU32,
 	) -> Result<u32, Error> {
+		let source: u8 = source.into();
+		let sink: u8 = sink.into();
 		let ret: Result<NullAndStringOr<'_, OneValue<_>>, oc_wasm_safe::error::Error> =
 			component_method(
 				self.invoker,
@@ -400,8 +418,8 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 				&self.address,
 				"transferItem",
 				Some(&FiveValues(
-					u8::from(source),
-					u8::from(sink),
+					source,
+					sink,
 					count,
 					source_slot.get(),
 					sink_slot.get(),
@@ -411,6 +429,38 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 		Ok(Locked::unpack_bad_parameters_with_message(ret, "invalid slot")?.0)
 	}
 
+	/// Moves fluids between two tanks.
+	///
+	/// The `count` value indicates the maximum number of millibuckets to move. The number of
+	/// millibuckets moved is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a transposer.
+	/// * [`Failed`](Error::Failed) is returned if there is not an accessible fluid tank on one of
+	///   the specified sides, or if there is not enough energy to perform the operation.
+	pub async fn transfer_fluid<SideType: Side>(
+		&mut self,
+		source: SideType,
+		sink: SideType,
+		count: u32,
+	) -> Result<u32, Error> {
+		let source: u8 = source.into();
+		let sink: u8 = sink.into();
+		let ret: Result<NullAndStringOr<'_, TwoValues<bool, u32>>, oc_wasm_safe::error::Error> =
+			component_method(
+				self.invoker,
+				self.buffer,
+				&self.address,
+				"transferFluid",
+				Some(&ThreeValues(source, sink, count)),
+			)
+			.await;
+		Ok(Locked::unpack_bad_parameters_with_message(ret, "invalid tank")?.1)
+	}
+
+	// WorldTankAnalytics
+
 	/// Returns the amount of fluid in a tank, in millibuckets.
 	///
 	/// The `tank` parameter ranges from 1 to the number of tanks in the target block.
@@ -422,16 +472,17 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   side or if the requested tank number is greater than the number of tanks.
 	pub async fn get_tank_level(
 		&mut self,
-		side: AbsoluteSide,
+		side: impl Side,
 		tank: NonZeroU32,
 	) -> Result<u32, Error> {
+		let side: u8 = side.into();
 		let ret: Result<NullAndStringOr<'_, OneValue<_>>, oc_wasm_safe::error::Error> =
 			component_method(
 				self.invoker,
 				self.buffer,
 				&self.address,
 				"getTankLevel",
-				Some(&TwoValues(u8::from(side), tank.get())),
+				Some(&TwoValues(side, tank.get())),
 			)
 			.await;
 		Ok(Locked::unpack_bad_parameters_with_message(ret, "invalid tank")?.0)
@@ -448,16 +499,17 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   side or if the requested tank number is greater than the number of tanks.
 	pub async fn get_tank_capacity(
 		&mut self,
-		side: AbsoluteSide,
+		side: impl Side,
 		tank: NonZeroU32,
 	) -> Result<u32, Error> {
+		let side: u8 = side.into();
 		let ret: Result<NullAndStringOr<'_, OneValue<_>>, oc_wasm_safe::error::Error> =
 			component_method(
 				self.invoker,
 				self.buffer,
 				&self.address,
 				"getTankCapacity",
-				Some(&TwoValues(u8::from(side), tank.get())),
+				Some(&TwoValues(side, tank.get())),
 			)
 			.await;
 		Ok(Locked::unpack_bad_parameters_with_message(ret, "invalid tank")?.0)
@@ -475,18 +527,19 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   full item information is disabled in the configuration file.
 	pub async fn get_fluid_in_tank(
 		self,
-		side: AbsoluteSide,
+		side: impl Side,
 		tank: NonZeroU32,
-	) -> Result<Option<Fluid<'buffer>>, Error> {
+	) -> Result<Option<FluidInTank<'buffer>>, Error> {
+		let side: u8 = side.into();
 		let ret: Result<
-			NullAndStringOr<'_, OneValue<OptionFluid<'buffer>>>,
+			NullAndStringOr<'_, OneValue<OptionFluidInTank<'buffer>>>,
 			oc_wasm_safe::error::Error,
 		> = component_method(
 			self.invoker,
 			self.buffer,
 			&self.address,
 			"getFluidInTank",
-			Some(&TwoValues(u8::from(side), tank.get())),
+			Some(&TwoValues(side, tank.get())),
 		)
 		.await;
 		let ret = Self::unpack_bad_parameters_with_message(ret, "invalid tank")?;
@@ -505,45 +558,356 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 	///   side or if reading full item information is disabled in the configuration file.
 	pub async fn get_fluids_in_tanks(
 		self,
-		side: AbsoluteSide,
-	) -> Result<Vec<Option<Fluid<'buffer>>>, Error> {
+		side: impl Side,
+	) -> Result<Vec<Option<FluidInTank<'buffer>>>, Error> {
+		let side: u8 = side.into();
 		let ret: NullAndStringOr<'_, OneValue<GetFluidsInTanksResult<'buffer>>> = component_method(
 			self.invoker,
 			self.buffer,
 			&self.address,
 			"getFluidInTank",
-			Some(&OneValue(u8::from(side))),
+			Some(&OneValue(side)),
 		)
 		.await?;
 		Ok(ret.into_result()?.0 .0)
 	}
 
-	/// Moves fluids between two tanks.
+	// TankInventoryControl
+
+	/// Returns the amount of fluid in a fluid container in the robot or drone’s internal
+	/// inventory.
 	///
-	/// The `count` value indicates the maximum number of millibuckets to move. The number of
-	/// millibuckets moved is returned.
+	/// The `slot` parameter ranges from 1 to the internal inventory size.
+	///
+	/// If the slot does not contain a fluid container (either because it contains a
+	/// non-fluid-container item or because it does not contain anything), `None` is returned. If
+	/// the slot contains an empty fluid container, `Some(0)` is returned.
 	///
 	/// # Errors
 	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
-	///   not a transposer.
-	/// * [`Failed`](Error::Failed) is returned if there is not an accessible fluid tank on one of
-	///   the specified sides, or if there is not enough energy to perform the operation.
-	pub async fn transfer_fluid(
+	///   not a tank controller upgrade installed in a robot or drone.
+	/// * [`Failed`](Error::Failed) is returned if the requested slot number is greater than the
+	///   inventory size.
+	pub async fn get_tank_level_in_slot(&mut self, slot: NonZeroU32) -> Result<Option<u32>, Error> {
+		let ret: NullAndStringOr<'_, OneValue<_>> = component_method(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getTankLevelInSlot",
+			Some(&OneValue(slot.get())),
+		)
+		.await?;
+		// Not-a-container is returned as (nil, string). Not-a-valid-inventory-slot is returned as
+		// an actual error.
+		match ret {
+			NullAndStringOr::Ok(n) => Ok(Some(n.0)),
+			NullAndStringOr::Err(_) => Ok(None),
+		}
+	}
+
+	/// Returns the amount of fluid in the fluid container in the currently selected slot of the
+	/// robot or drone’s internal inventory.
+	///
+	/// If the slot does not contain a fluid container (either because it contains a
+	/// non-fluid-container item or because it does not contain anything), `None` is returned. If
+	/// the slot contains an empty fluid container, `Some(0)` is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	pub async fn get_tank_level_in_selected_slot(&mut self) -> Result<Option<u32>, Error> {
+		let ret: NullAndStringOr<'_, OneValue<_>> = component_method::<(), _>(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getTankLevelInSlot",
+			None,
+		)
+		.await?;
+		match ret {
+			NullAndStringOr::Ok(n) => Ok(Some(n.0)),
+			NullAndStringOr::Err(_) => Ok(None),
+		}
+	}
+
+	/// Returns the total size of a fluid container in the robot or drone’s internal inventory.
+	///
+	/// The `slot` parameter ranges from 1 to the internal inventory size.
+	///
+	/// If the slot does not contain a fluid container (either because it contains a
+	/// non-fluid-container item or because it does not contain anything), `None` is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	/// * [`Failed`](Error::Failed) is returned if the requested slot number is greater than the
+	///   inventory size.
+	pub async fn get_tank_capacity_in_slot(
 		&mut self,
-		source: AbsoluteSide,
-		sink: AbsoluteSide,
-		count: u32,
-	) -> Result<u32, Error> {
-		let ret: Result<NullAndStringOr<'_, TwoValues<bool, u32>>, oc_wasm_safe::error::Error> =
-			component_method(
-				self.invoker,
-				self.buffer,
-				&self.address,
-				"transferFluid",
-				Some(&ThreeValues(u8::from(source), u8::from(sink), count)),
-			)
-			.await;
-		Ok(Locked::unpack_bad_parameters_with_message(ret, "invalid tank")?.1)
+		slot: NonZeroU32,
+	) -> Result<Option<u32>, Error> {
+		let ret: NullAndStringOr<'_, OneValue<_>> = component_method(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getTankCapacityInSlot",
+			Some(&OneValue(slot.get())),
+		)
+		.await?;
+		// Not-a-container is returned as (nil, string). Not-a-valid-inventory-slot is returned as
+		// an actual error.
+		match ret {
+			NullAndStringOr::Ok(n) => Ok(Some(n.0)),
+			NullAndStringOr::Err(_) => Ok(None),
+		}
+	}
+
+	/// Returns the total size of the fluid container in the currently selected slot of the robot
+	/// or drone’s internal inventory.
+	///
+	/// If the slot does not contain a fluid container (either because it contains a
+	/// non-fluid-container item or because it does not contain anything), `None` is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	pub async fn get_tank_capacity_in_selected_slot(&mut self) -> Result<Option<u32>, Error> {
+		let ret: NullAndStringOr<'_, OneValue<_>> = component_method::<(), _>(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getTankCapacityInSlot",
+			None,
+		)
+		.await?;
+		match ret {
+			NullAndStringOr::Ok(n) => Ok(Some(n.0)),
+			NullAndStringOr::Err(_) => Ok(None),
+		}
+	}
+
+	/// Returns information about the fluid in a fluid container in the robot or drone’s internal
+	/// inventory.
+	///
+	/// The `slot` parameter ranges from 1 to the internal inventory size.
+	///
+	/// If the slot does not contain fluid (either because it contains a non-fluid-container item,
+	/// because it does not contain anything, or because it contains a fluid container item without
+	/// any fluid), `None` is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	/// * [`Failed`](Error::Failed) is returned if the requested slot number is greater than the
+	///   inventory size.
+	pub async fn get_fluid_in_tank_in_slot(
+		self,
+		slot: NonZeroU32,
+	) -> Result<Option<Fluid<'buffer>>, Error> {
+		let ret: NullAndStringOr<'_, OneValue<_>> = component_method(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getFluidInTankInSlot",
+			Some(&OneValue(slot.get())),
+		)
+		.await?;
+		// Not-a-container is returned as (nil, string). Not-a-valid-inventory-slot is returned as
+		// an actual error. Empty container as returned as (nil) without a string.
+		match ret {
+			NullAndStringOr::Ok(n) => Ok(n.0),
+			NullAndStringOr::Err(_) => Ok(None),
+		}
+	}
+
+	/// Returns information about the fluid in the fluid container in the selected slot of the
+	/// robot or drone’s internal inventory.
+	///
+	/// If the slot does not contain fluid (either because it contains a non-fluid-container item,
+	/// because it does not contain anything, or because it contains a fluid container item without
+	/// any fluid), `None` is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	pub async fn get_fluid_in_tank_in_selected_slot(self) -> Result<Option<Fluid<'buffer>>, Error> {
+		let ret: NullAndStringOr<'_, OneValue<_>> = component_method::<(), _>(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getFluidInTankInSlot",
+			None,
+		)
+		.await?;
+		match ret {
+			NullAndStringOr::Ok(n) => Ok(n.0),
+			NullAndStringOr::Err(_) => Ok(None),
+		}
+	}
+
+	/// Returns information about the fluid in the robot or drone’s specified internal tank.
+	///
+	/// If the tank is empty, `None` is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	/// * [`Failed`](Error::Failed) is returned if the specified tank number is greater than the
+	///   number of internal tanks in the robot or drone.
+	pub async fn get_fluid_in_internal_tank(
+		self,
+		tank: NonZeroU32,
+	) -> Result<Option<Fluid<'buffer>>, Error> {
+		Ok(component_method::<_, OneValue<_>>(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getFluidInInternalTank",
+			Some(&OneValue(tank.get())),
+		)
+		.await?
+		.0)
+	}
+
+	/// Returns information about the fluid in the robot or drone’s currently selected internal
+	/// tank.
+	///
+	/// If the tank is empty, `None` is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	pub async fn get_fluid_in_selected_internal_tank(
+		self,
+	) -> Result<Option<Fluid<'buffer>>, Error> {
+		Ok(component_method::<(), OneValue<_>>(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getFluidInInternalTank",
+			None,
+		)
+		.await?
+		.0)
+	}
+
+	/// Moves fluid from a fluid container in the robot or drone’s currently selected inventory
+	/// slot into the robot or drone’s currently selected internal tank.
+	///
+	/// On success, the amount of fluid moved is returned. For certain types of source containers,
+	/// this may be larger than `amount`.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	/// * [`Failed`](Error::Failed) is returned if the robot does not contain any tanks, the
+	///   selected internal tank is full, the selected item is not a container, the source
+	///   container is empty, the internal tank and source container contain different types of
+	///   fluid, or the source container cannot be partly drained (e.g. a bucket) and the complete
+	///   amount cannot be moved (either because `amount` is too small or because there is not
+	///   enough space in the destination tank).
+	pub async fn drain(&mut self, amount: NonZeroU32) -> Result<u32, Error> {
+		self.drain_or_fill(amount, "drain").await
+	}
+
+	/// Moves fluid from the robot or drone’s currently selected internal tank into a fluid
+	/// container in the robot or drone’s currently selected inventory slot.
+	///
+	/// On success, the amount of fluid moved is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	/// * [`Failed`](Error::Failed) is returned if the robot does not contain any tanks, the
+	///   selected internal tank is empty, the selected item is not a container, the destination
+	///   container is full, the internal tank and destination container contain different types of
+	///   fluid, or the destination container cannot be partly filled (e.g. a bucket) and the
+	///   complete amount cannot be moved (either because `amount` is too small or because there is
+	///   not enough fluid in the source tank).
+	pub async fn fill(&mut self, amount: NonZeroU32) -> Result<u32, Error> {
+		self.drain_or_fill(amount, "fill").await
+	}
+
+	// InventoryAnalytics
+
+	/// Returns the item stack in a robot or drone’s internal inventory slot.
+	///
+	/// The `slot` parameter ranges from 1 to the inventory size. If the slot does not contain any
+	/// items, `None` is returned.
+	///
+	/// The strings in the returned item stack point into, and therefore retain ownership of, the
+	/// scratch buffer. Consequently, the `Locked` is consumed and cannot be reused.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not an inventory controller upgrade installed in a robot or drone.
+	/// * [`Failed`](Error::Failed) is returned if the requested slot number is greater than the
+	///   inventory size, or if reading full item information is disabled in the configuration
+	///   file.
+	pub async fn get_stack_in_internal_slot(
+		self,
+		slot: NonZeroU32,
+	) -> Result<Option<ItemStack<'buffer>>, Error> {
+		let ret: NullAndStringOr<'_, OneValue<_>> = component_method(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getStackInInternalSlot",
+			Some(&OneValue(slot)),
+		)
+		.await?;
+		Ok(ret.into_result()?.0)
+	}
+
+	/// Returns the item stack in the robot or drone’s currently selected internal inventory slot.
+	///
+	/// The strings in the returned item stack point into, and therefore retain ownership of, the
+	/// scratch buffer. Consequently, the `Locked` is consumed and cannot be reused.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not an inventory controller upgrade installed in a robot or drone.
+	/// * [`Failed`](Error::Failed) is returned if reading full item information is disabled in the
+	///   configuration file.
+	pub async fn get_stack_in_selected_internal_slot(
+		self,
+	) -> Result<Option<ItemStack<'buffer>>, Error> {
+		let ret: NullAndStringOr<'_, OneValue<_>> = component_method::<(), _>(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"getStackInInternalSlot",
+			None,
+		)
+		.await?;
+		Ok(ret.into_result()?.0)
+	}
+
+	/// Checks whether a robot or drone’s internal inventory slot contains an
+	/// ore-dictionary-equivalent item to the currently selected internal inventory slot.
+	///
+	/// The `slot` parameter ranges from 1 to the inventory size.
+	///
+	/// Two empty slots are considered equivalent. An empty slot and a populated slot are
+	/// considered non-equivalent. A slot is considered equivalent to itself. Otherwise, two slots
+	/// are considered equivalent if and only if the items in both slots have at least one ore
+	/// dictionary “ore ID” entry in common.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   neither a transposer nor an inventory controller upgrade.
+	/// * [`Failed`](Error::Failed) is returned if the requested slot number is greater than the
+	///   inventory size.
+	pub async fn is_equivalent_to(&mut self, slot: NonZeroU32) -> Result<bool, Error> {
+		Ok(component_method::<_, OneValue<_>>(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			"isEquivalentTo",
+			Some(&OneValue(slot.get())),
+		)
+		.await?
+		.0)
 	}
 
 	/// Makes a method call that accepts one or more slot parameters and converts a
@@ -590,6 +954,29 @@ impl<'invoker, 'buffer> Locked<'invoker, 'buffer> {
 			Err(e) => Err(e.into()),
 		}
 	}
+
+	/// Implements the `drain` and `fill` functions.
+	///
+	/// On success, the amount of fluid moved is returned.
+	///
+	/// # Errors
+	/// * [`BadComponent`](Error::BadComponent) is returned if the component does not exist or is
+	///   not a tank controller upgrade installed in a robot or drone.
+	/// * [`Failed`](Error::Failed) is returned if the operation failed.
+	async fn drain_or_fill(&mut self, amount: NonZeroU32, method: &str) -> Result<u32, Error> {
+		let ret: NullAndStringOr<'_, TwoValues<bool, u32>> = component_method(
+			self.invoker,
+			self.buffer,
+			&self.address,
+			method,
+			Some(&OneValue(amount.get())),
+		)
+		.await?;
+		match ret {
+			NullAndStringOr::Ok(x) => Ok(x.1),
+			NullAndStringOr::Err(e) => Err(Error::Failed(e.to_owned())),
+		}
+	}
 }
 
 /// Information about an item stack.
@@ -893,7 +1280,7 @@ impl<'snapshot, 'invoker, 'buffer> LockedSnapshot<'snapshot, 'invoker, 'buffer>
 /// The `'buffer` lifetime is the lifetime of the buffer holding strings to which the object
 /// refers.
 #[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-pub struct Fluid<'buffer> {
+pub struct FluidInTank<'buffer> {
 	/// The internal (Minecraft system) name of the item.
 	///
 	/// For example, this might be `water`.
@@ -914,7 +1301,7 @@ pub struct Fluid<'buffer> {
 	pub has_tag: bool,
 }
 
-impl<'buffer> Decode<'buffer> for Fluid<'buffer> {
+impl<'buffer> Decode<'buffer> for FluidInTank<'buffer> {
 	fn decode(d: &mut Decoder<'buffer>) -> Result<Self, minicbor::decode::Error> {
 		// The CBOR fits in memory, so it must be <2³² elements.
 		#[allow(clippy::cast_possible_truncation)]
@@ -956,18 +1343,89 @@ impl<'buffer> Decode<'buffer> for Fluid<'buffer> {
 	}
 }
 
+/// Information about a fluid.
+///
+/// The `'buffer` lifetime is the lifetime of the buffer holding strings to which the object
+/// refers.
+#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+pub struct Fluid<'buffer> {
+	/// The internal (Minecraft system) name of the item.
+	///
+	/// For example, this might be `water`.
+	pub name: &'buffer str,
+
+	/// The human-readable name of the item.
+	///
+	/// For example, this might be `Water`.
+	pub label: &'buffer str,
+
+	/// The number of millibuckets of fluid in the container.
+	pub amount: u32,
+
+	/// Whether the fluid has extra NBT data attached.
+	pub has_tag: bool,
+}
+
+impl<'buffer> Decode<'buffer> for Fluid<'buffer> {
+	fn decode(d: &mut Decoder<'buffer>) -> Result<Self, minicbor::decode::Error> {
+		// The CBOR fits in memory, so it must be <2³² elements.
+		#[allow(clippy::cast_possible_truncation)]
+		let len = d.map()?.ok_or(minicbor::decode::Error::Message(""))? as usize;
+		let mut name: Option<_> = None;
+		let mut label: Option<_> = None;
+		let mut amount: Option<_> = None;
+		let mut has_tag: Option<_> = None;
+		for _ in 0..len {
+			let key = d.str()?;
+			match key {
+				"name" => name = Some(d.str()?),
+				"label" => label = Some(d.str()?),
+				"amount" => amount = Some(d.u32()?),
+				"hasTag" => has_tag = Some(d.bool()?),
+				_ => return Err(minicbor::decode::Error::Message("")),
+			}
+		}
+		if let Some(name) = name {
+			if let Some(label) = label {
+				if let Some(amount) = amount {
+					if let Some(has_tag) = has_tag {
+						return Ok(Self {
+							name,
+							label,
+							amount,
+							has_tag,
+						});
+					}
+				}
+			}
+		}
+		Err(minicbor::decode::Error::Message(""))
+	}
+}
+
+impl<'buffer> From<FluidInTank<'buffer>> for Fluid<'buffer> {
+	fn from(src: FluidInTank<'buffer>) -> Self {
+		Self {
+			name: src.name,
+			label: src.label,
+			amount: src.amount,
+			has_tag: src.has_tag,
+		}
+	}
+}
+
 /// Information about a fluid in a tank which may or may not exist.
 ///
-/// This type exists, rather than just using `Option<Fluid>` directly, because `Option` has a
+/// This type exists, rather than just using `Option<FluidInTank>` directly, because `Option` has a
 /// blanket `Decode` implementation, and we need a different implementation which also maps a
 /// non-null map containing a skeleton subset of keys to `None`.
 ///
 /// The `'buffer` lifetime is the lifetime of the buffer holding strings to which the object
 /// refers.
 #[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-struct OptionFluid<'buffer>(Option<Fluid<'buffer>>);
+struct OptionFluidInTank<'buffer>(Option<FluidInTank<'buffer>>);
 
-impl<'buffer> Decode<'buffer> for OptionFluid<'buffer> {
+impl<'buffer> Decode<'buffer> for OptionFluidInTank<'buffer> {
 	fn decode(d: &mut Decoder<'buffer>) -> Result<Self, minicbor::decode::Error> {
 		if d.datatype()? == minicbor::data::Type::Null {
 			// Null is not used by OpenComputers AFAIK, but for future-proofing should probably map
@@ -1012,14 +1470,14 @@ impl<'buffer> Decode<'buffer> for OptionFluid<'buffer> {
 	}
 }
 
-/// A vector of `Option<Fluid>` objects that is decoded from a CBOR array.
+/// A vector of `Option<FluidInTank>` objects that is decoded from a CBOR array.
 ///
-/// Each element is decoded as an `OptionFluid` rather than an `Option<Fluid>`.
+/// Each element is decoded as an `OptionFluidInTank` rather than an `Option<FluidInTank>`.
 ///
 /// The `'buffer` lifetime is the lifetime of the buffer holding strings to which the objects
 /// refer.
 #[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-struct GetFluidsInTanksResult<'buffer>(Vec<Option<Fluid<'buffer>>>);
+struct GetFluidsInTanksResult<'buffer>(Vec<Option<FluidInTank<'buffer>>>);
 
 impl<'buffer> Decode<'buffer> for GetFluidsInTanksResult<'buffer> {
 	fn decode(d: &mut Decoder<'buffer>) -> Result<Self, minicbor::decode::Error> {
@@ -1029,7 +1487,7 @@ impl<'buffer> Decode<'buffer> for GetFluidsInTanksResult<'buffer> {
 		let len = len.ok_or(minicbor::decode::Error::Message(""))? as usize;
 		let mut ret = Vec::with_capacity(len);
 		for _ in 0..len {
-			ret.push(d.decode::<OptionFluid<'buffer>>()?.0);
+			ret.push(d.decode::<OptionFluidInTank<'buffer>>()?.0);
 		}
 		Ok(Self(ret))
 	}
